# See the loader file for license and author info

require 'sketchup.rb'
module CLF_Extensions_NS

  module CLF_Color_by_Z
    # Define the method.  Set the model and the selection. Add a line to the info bar when the user is running the script.
    def self.colorbyz
      model = Sketchup.active_model
      clselection = model.selection
      result = Sketchup.set_status_text( "Color by Z! by Chris Fullmer", SB_PROMPT )
      #  This sets what entities will be used - either the selection set if it exists or all entities in the model.  If there is nothing selected, the user gets a warning message.
      if clselection.empty?
        UI.messagebox "You have nothing selected.  This will color all faces that are not grouped."
        entities = model.active_entities
      else
        entities = model.selection
      end
      # Define some variables and arrays
      faces = []
      face_high = 0
      face_low = 0
      face_low_helper = 0
      values = [0,0,0]
      prompts = ["R:", "G:", "B:"]
      
      # Get the SketchUp version and start the operation for the Undo stack.  Version 7 works differently than lower versions so a version check needs to be put here
      a = Sketchup.version
      b = a[0,1].to_i
      if b > 6
        model.start_operation("Color by Z", true)
      else
        model.start_operation("Color by Z")
      end
      # runs through the chosen entities.  makes as array of faces for later use.  During that process, finds the face with the highest Z value and the face with the lowest Z value.
      entities.each do |a|
        if a.typename == ( "Face" )
          facez = a.bounds.center[2]
          if face_low_helper == 0 
            face_low = facez
            face_low_helper = 1
          end 
          if facez > face_high
            face_high = facez
          end
          if a.bounds.center[2].to_l < face_low
            face_low = facez
          end
          faces.push a
        end
      end
      # Set these variables so that the menus run.  After the menu runs, the variable is set to 1.  But if there is an error in the user input, the value gets
      # put back to 0, thus making the menu run again after the error message is sent to the user.
      retrymenu1 = 0
      retrymenu2 = 0
      # Menu 1
      while retrymenu1 == 0
        results1 = inputbox( prompts, values, "Highest Z value color (in RGB)" )
        retrymenu1 = 1
        results1.each do |a|
          if a < 0 # Start the error checking and error messages
            UI.messagebox "All values must be greater than zero"
            retrymenu1 = 0
          end
          if a > 255
            UI.messagebox "All values must be less than 255"
            retrymenu1 = 0
          end
        end
      end
      # Menu 2
      values = [255, 255, 255]
      while retrymenu2 == 0
        results2 = inputbox( prompts, values, "Lowest Z value color (in RGB)" )
        retrymenu2 = 1
        results2.each do |a|
          if a < 0 # Start the error checking and error messages
            UI.messagebox "All values must be greater than zero"
            retrymenu2 = 0
          end
          if a > 255
            UI.messagebox "All values must be less than 255"
            retrymenu2 = 0
          end
        end
      end
      
      #  This section gets differences in RGB's, face heights, and anything else it needs to figure out what color to each face.
      # notice it is running through each face that I previously separated out into the faces array.
      rdif = results1[0] - results2[0]
      gdif = results1[1] - results2[1]
      bdif = results1[2] - results2[2]
      face_dif = face_high - face_low
      rbase = results2[0]
      gbase = results2[1]
      bbase = results2[2]
      
      #count = 0
      #pb = ProgressBar.new(faces.length, "#{faces.length} to be colored." )
      
      faces.each do |a|
        cp = a.bounds.center[2] - face_low
        #pb.update(count)
        if rdif == 0
          r = results1[0]
        else
          r = ((cp*100 / face_dif) * rdif / 100) + rbase
        end
        if gdif == 0
          g = results1[1]
        else
          g = ((cp*100 / face_dif) * gdif / 100) + gbase
        end
        if bdif == 0
          b = results1[2]
        else
          b = ((cp*100 / face_dif) * bdif / 100) + bbase
        end
        a.material = [r.to_i, g.to_i ,b.to_i]
        a.back_material = [r.to_i, g.to_i ,b.to_i]
        #count = count + 1
      end		
      model.commit_operation
    end
  end
end